from edc_auth.constants import (
    AUDITOR_ROLE,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    NURSE_ROLE,
)
from edc_auth.site_auths import site_auths
from edc_auth.utils import remove_default_model_permissions_from_edc_permissions

from .auth_objects import (
    ae_codenames,
    ae_dashboard_tuples,
    ae_navbar_tuples,
    tmg_codenames,
    tmg_dashboard_tuples,
    tmg_navbar_tuples,
)
from .constants import AE, AE_REVIEW, AE_ROLE, AE_SUPER, TMG, TMG_REVIEW, TMG_ROLE

site_auths.add_post_update_func(
    "edc_adverse_event", remove_default_model_permissions_from_edc_permissions
)

permissions_model = "edc_adverse_event.edcpermissions"

# custom perms
site_auths.add_custom_permissions_tuples(
    model=permissions_model, codename_tuples=ae_dashboard_tuples
)
site_auths.add_custom_permissions_tuples(
    model=permissions_model, codename_tuples=tmg_dashboard_tuples
)

site_auths.add_custom_permissions_tuples(
    model=permissions_model, codename_tuples=tmg_navbar_tuples
)
site_auths.add_custom_permissions_tuples(
    model=permissions_model, codename_tuples=ae_navbar_tuples
)

# groups
site_auths.add_group(*ae_codenames, name=AE, no_delete=True)
site_auths.update_group(
    *[c[0] for c in ae_dashboard_tuples], *[c[0] for c in ae_navbar_tuples], name=AE
)

site_auths.add_group(*ae_codenames, name=AE_SUPER)
site_auths.update_group(
    *[c[0] for c in ae_dashboard_tuples], *[c[0] for c in ae_navbar_tuples], name=AE_SUPER
)

site_auths.add_group(*ae_codenames, name=AE_REVIEW, view_only=True)
site_auths.update_group(
    *[c[0] for c in ae_dashboard_tuples], *[c[0] for c in ae_navbar_tuples], name=AE_REVIEW
)

site_auths.add_group(*tmg_codenames, name=TMG)
site_auths.update_group(
    *[c[0] for c in tmg_dashboard_tuples], *[c[0] for c in tmg_navbar_tuples], name=TMG
)

site_auths.add_group(*tmg_codenames, name=TMG_REVIEW, view_only=True)
site_auths.update_group(
    *[c[0] for c in tmg_dashboard_tuples], *[c[0] for c in tmg_navbar_tuples], name=TMG_REVIEW
)

# add roles
site_auths.add_role(AE, name=AE_ROLE)
site_auths.add_role(AE_REVIEW, TMG, name=TMG_ROLE)


site_auths.update_role(AE, name=CLINICIAN_ROLE)
site_auths.update_role(AE, name=NURSE_ROLE)
site_auths.update_role(AE_REVIEW, TMG_REVIEW, name=AUDITOR_ROLE)
site_auths.update_role(AE_SUPER, name=CLINICIAN_SUPER_ROLE)



from edc_constants.constants import (
    DEAD,
    MALIGNANCY,
    NOT_APPLICABLE,
    OTHER,
    TUBERCULOSIS,
    UNKNOWN,
)
from edc_constants.disease_constants import (
    ANAEMIA,
    BACTERAEMIA,
    BACTERIAL_PNEUMONIA,
    NEUTROPAENIA,
    PNEUMONIA,
    RENAL_IMPAIRMENT,
    THROMBOCYTOPENIA,
)

list_data = {
    "edc_adverse_event.causeofdeath": [
        (BACTERAEMIA, "Bacteraemia"),
        (BACTERIAL_PNEUMONIA, "Bacterial pneumonia"),
        (MALIGNANCY, "Malignancy"),
        ("art_toxicity", "ART toxicity"),
        ("IRIS_non_CM", "IRIS non-CM"),
        ("diarrhea_wasting", "Diarrhea/wasting"),
        (UNKNOWN, "Unknown"),
        (OTHER, "Other"),
    ],
    "edc_adverse_event.aeclassification": [
        (ANAEMIA, "Anaemia"),
        ("bacteraemia/sepsis", "Bacteraemia/Sepsis"),
        ("CM_IRIS", "CM IRIS"),
        ("diarrhoea", "Diarrhoea"),
        ("hypokalaemia", "Hypokalaemia"),
        (NEUTROPAENIA, "Neutropaenia"),
        (PNEUMONIA, "Pneumonia"),
        (RENAL_IMPAIRMENT, "Renal impairment"),
        ("respiratory_distress", "Respiratory distress"),
        (TUBERCULOSIS, "TB"),
        (THROMBOCYTOPENIA, "Thrombocytopenia"),
        ("thrombophlebitis", "Thrombophlebitis"),
        (OTHER, "Other"),
    ],
    "edc_adverse_event.saereason": [
        (NOT_APPLICABLE, "Not applicable"),
        (DEAD, "Death"),
        ("life_threatening", "Life-threatening"),
        ("significant_disability", "Significant disability"),
        (
            "in-patient_hospitalization",
            (
                "In-patient hospitalization or prolongation "
                "(17 or more days from study inclusion)"
            ),
        ),
        (
            "medically_important_event",
            "Medically important event (e.g. Severe thrombophlebitis, Bacteraemia, "
            "recurrence of symptoms not requiring admission, Hospital acquired "
            "pneumonia)",
        ),
    ],
}



#!/usr/bin/env python
import sys
from pathlib import Path

from dateutil.relativedelta import relativedelta
from edc_test_settings.default_test_settings import DefaultTestSettings
from edc_utils import get_utcnow

app_name = "edc_adverse_event"
base_dir = Path(__file__).absolute().parent.parent.parent

project_settings = DefaultTestSettings(
    calling_file=__file__,
    BASE_DIR=base_dir,
    APP_NAME=app_name,
    ETC_DIR=str(base_dir / app_name / "tests" / "etc"),
    HOLIDAY_FILE=base_dir / app_name / "tests" / "etc" / "holidays.csv",
    SILENCED_SYSTEM_CHECKS=["sites.E101", "edc_navbar.E002", "edc_navbar.E003"],
    SUBJECT_VISIT_MODEL="adverse_event_app.subjectvisit",
    ADVERSE_EVENT_APP_LABEL="adverse_event_app",
    ADVERSE_EVENT_ADMIN_SITE="adverse_event_app_admin",
    EMAIL_ENABLED=True,
    EMAIL_CONTACTS={
        "ae_reports": "aereports@example.com",
        "data_manager": "data_manager@example.com",
        "tmg": "tmg@example.com",
    },
    EDC_PROTOCOL_STUDY_OPEN_DATETIME=get_utcnow().replace(
        microsecond=0, second=0, minute=0, hour=0
    )
    - relativedelta(years=2),
    EDC_PROTOCOL_STUDY_CLOSE_DATETIME=get_utcnow().replace(
        microsecond=999999, second=59, minute=59, hour=11
    )
    + relativedelta(years=2),
    EDC_NAVBAR_DEFAULT=app_name,
    EDC_AUTH_SKIP_SITE_AUTHS=True,
    EDC_AUTH_SKIP_AUTH_UPDATER=True,
    EDC_SITES_REGISTER_DEFAULT=True,
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_crypto_fields.apps.AppConfig",
        "multisite.apps.AppConfig",
        "edc_auth.apps.AppConfig",
        "edc_action_item.apps.AppConfig",
        "edc_adverse_event.apps.AppConfig",
        "edc_appointment.apps.AppConfig",
        "edc_consent.apps.AppConfig",
        "edc_crf.apps.AppConfig",
        "edc_data_manager.apps.AppConfig",
        "edc_dashboard.apps.AppConfig",
        "edc_device.apps.AppConfig",
        "edc_facility.apps.AppConfig",
        "edc_form_runners.apps.AppConfig",
        "edc_identifier.apps.AppConfig",
        "edc_lab.apps.AppConfig",
        "edc_list_data.apps.AppConfig",
        "edc_locator.apps.AppConfig",
        "edc_metadata.apps.AppConfig",
        "edc_navbar.apps.AppConfig",
        "edc_notification.apps.AppConfig",
        "edc_offstudy.apps.AppConfig",
        "edc_prn.apps.AppConfig",
        "edc_protocol.apps.AppConfig",
        "edc_randomization.apps.AppConfig",
        "edc_registration.apps.AppConfig",
        "edc_review_dashboard.apps.AppConfig",
        "edc_sites.apps.AppConfig",
        "edc_subject_dashboard.apps.AppConfig",
        "edc_timepoint.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "adverse_event_app.apps.AppConfig",
        "edc_appconfig.apps.AppConfig",
    ],
    add_dashboard_middleware=True,
).settings


for k, v in project_settings.items():
    setattr(sys.modules[__name__], k, v)



from django.db import models
from edc_constants.choices import YES_NO, YES_NO_UNKNOWN
from edc_model_fields.fields.other_charfield import OtherCharField

from edc_adverse_event.models import AeActionClassification


class AeInitialAeModelMixin(models.Model):
    ae_study_relation_possibility = models.CharField(
        verbose_name="Is the incident related to the patient involvement in the study?",
        max_length=10,
        choices=YES_NO_UNKNOWN,
    )

    ae_cause = models.CharField(
        verbose_name=(
            "Has a reason other than the specified study drug been "
            "identified as the cause of the event(s)?"
        ),
        choices=YES_NO,
        max_length=5,
    )

    ae_cause_other = OtherCharField(
        verbose_name='If "Yes", specify', max_length=250, blank=True, null=True
    )

    ae_treatment = models.TextField(verbose_name="Specify action taken for treatment of AE:")

    ae_action_classification = models.ForeignKey(
        AeActionClassification,
        on_delete=models.PROTECT,
        verbose_name="Classification of action taken",
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True



from django import forms

from ..modelform_mixins import AeFollowupModelFormMixin, AeModelFormMixin
from ..utils import get_ae_model


class AeFollowupForm(AeFollowupModelFormMixin, AeModelFormMixin, forms.ModelForm):
    class Meta(AeFollowupModelFormMixin.Meta):
        model = get_ae_model("aefollowup")
        fields = "__all__"



from .death_report_form_validator import DeathReportFormValidatorMixin
from .requires_death_report_form_validator_mixin import (
    BaseRequiresDeathReportFormValidatorMixin,
    RequiresDeathReportFormValidatorMixin,
)



from django.conf import settings

ADVERSE_EVENT_APP_LABEL = getattr(settings, "ADVERSE_EVENT_APP_LABEL", "edc_adverse_event")
ADVERSE_EVENT_ADMIN_SITE = getattr(
    settings, "ADVERSE_EVENT_ADMIN_SITE", "edc_adverse_event_admin"
)
AESI_ACTION = "submit-aesi-report"
AE_FOLLOWUP_ACTION = "submit-ae-followup-report"
AE_INITIAL_ACTION = "submit-initial-ae-report"
AE_SUSAR_ACTION = "submit-ae-susar-report"
AE_TMG_ACTION = "submit-ae-tmg-report"
AE_WITHDRAWN = "ae_withdrawn"
CONTINUING_UPDATE = "continuing/update"
DEATH_REPORT_ACTION = "submit-death-report"
DEATH_REPORT_NOT_FOUND = "DEATH_REPORT_NOT_FOUND"
DEATH_REPORT_TMG_ACTION = "submit-death-report-tmg"
DEATH_REPORT_TMG_SECOND_ACTION = "submit-death-report-tmg-2nd"
DEFINITELY_RELATED = "definitely_related"
DISCHARGED = "discharged"
HOSPITALIZATION_ACTION = "submit-hospitalization-report"
INPATIENT = "inpatient"
NOT_RECOVERED = "not_recovered"
NOT_RELATED = "not_related"
POSSIBLY_RELATED = "possibly_related"
PROBABLY_RELATED = "probably_related"
RECOVERED = "recovered"
RECOVERED_WITH_SEQUELAE = "recovered_with_sequelae"
RECOVERING = "recovering"
STUDY_TERMINATION_CONCLUSION_ACTION = "submit-study-termination-conclusion"
UNLIKELY_RELATED = "unlikely_related"

# roles
AE = "AE"
AE_REVIEW = "AE_REVIEW"
AE_ROLE = "ae_role"
AE_SUPER = "AE_SUPER"
TMG = "TMG"
TMG_REVIEW = "TMG_REVIEW"
TMG_ROLE = "tmg"



from typing import Any

from django.urls import reverse
from django.views.generic import TemplateView
from edc_dashboard.url_names import url_names
from edc_dashboard.view_mixins import EdcViewMixin, UrlRequestContextMixin
from edc_navbar import NavbarViewMixin

from ..utils import get_adverse_event_admin_site, get_adverse_event_app_label


class AeHomeView(UrlRequestContextMixin, EdcViewMixin, NavbarViewMixin, TemplateView):
    ae_listboard_url = "ae_listboard_url"
    death_report_listboard_url = "death_report_listboard_url"
    navbar_selected_item = "ae_home"
    template_name = "edc_adverse_event/ae/ae_home.html"
    url_name = "ae_home_url"

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        app_list_url = f"{get_adverse_event_admin_site()}:app_list"
        ae_initial_changelist_url = reverse(
            f"{get_adverse_event_admin_site()}:{get_adverse_event_app_label()}_"
            f"aeinitial_changelist"
        )
        death_report_changelist_url = reverse(
            f"{get_adverse_event_admin_site()}:{get_adverse_event_app_label()}_"
            f"deathreport_changelist"
        )
        ae_listboard_url = url_names.get(self.ae_listboard_url)
        death_report_listboard_url = url_names.get(self.death_report_listboard_url)
        kwargs.update(
            ADVERSE_EVENT_ADMIN_SITE=get_adverse_event_admin_site(),
            ADVERSE_EVENT_APP_LABEL=get_adverse_event_app_label(),
            app_list_url=app_list_url,
            ae_listboard_url=ae_listboard_url,
            ae_initial_changelist_url=ae_initial_changelist_url,
            death_report_changelist_url=death_report_changelist_url,
            death_report_listboard_url=death_report_listboard_url,
            **self.add_url_to_context(new_key="ae_home_url", existing_key=self.url_name),
        )
        return super().get_context_data(**kwargs)



from django import forms

from ..modelform_mixins import AeModelFormMixin, AeSusarModelFormMixin
from ..utils import get_ae_model


class AeSusarForm(AeSusarModelFormMixin, AeModelFormMixin, forms.ModelForm):
    class Meta(AeSusarModelFormMixin.Meta):
        model = get_ae_model("aesusar")
        fields = "__all__"



from edc_constants.constants import DEAD, NOT_APPLICABLE, OTHER, UNKNOWN

list_data = {
    "edc_adverse_event.aeclassification": [
        ("adr", "Adverse drug reaction (ADR)"),
        (OTHER, "Other"),
    ],
    "edc_adverse_event.saereason": [
        (NOT_APPLICABLE, "Not applicable"),
        (DEAD, "Death"),
        ("life_threatening", "Life-threatening"),
        ("significant_disability", "Significant disability"),
        (
            "in-patient_hospitalization",
            "In-patient hospitalization or prolongation ",
        ),
        (
            "medically_important_event",
            "Medically important event",
        ),
    ],
    "edc_adverse_event.causeofdeath": [
        (UNKNOWN, "Unknown"),
        (OTHER, "Other"),
    ],
    "edc_adverse_event.aeactionclassification": [
        (UNKNOWN, "Unknown"),
        (OTHER, "Other"),
    ],
}



from edc_action_item import site_action_items
from edc_action_item.action_with_notification import ActionWithNotification
from edc_constants.constants import HIGH_PRIORITY

from edc_adverse_event.action_items import (
    AeFollowupAction,
    AeInitialAction,
    AeSusarAction,
    AeTmgAction,
    DeathReportAction,
    DeathReportTmgAction,
    DeathReportTmgSecondAction,
)
from edc_adverse_event.constants import (
    AE_FOLLOWUP_ACTION,
    DEATH_REPORT_ACTION,
    STUDY_TERMINATION_CONCLUSION_ACTION,
)


class StudyTerminationConclusionAction(ActionWithNotification):
    name = STUDY_TERMINATION_CONCLUSION_ACTION
    display_name = "Submit Study Termination/Conclusion Report"
    notification_display_name = "Study Termination/Conclusion Report"
    parent_action_names = [
        DEATH_REPORT_ACTION,
        AE_FOLLOWUP_ACTION,
    ]
    reference_model = "adverse_event_app.studyterminationconclusion"
    show_link_to_changelist = True
    admin_site_name = "adverse_event_app_admin"
    priority = HIGH_PRIORITY


site_action_items.register(AeFollowupAction)
site_action_items.register(AeInitialAction)
site_action_items.register(AeSusarAction)
site_action_items.register(AeTmgAction)
site_action_items.register(DeathReportAction)
site_action_items.register(DeathReportTmgAction)
site_action_items.register(DeathReportTmgSecondAction)
site_action_items.register(StudyTerminationConclusionAction)



from edc_constants.constants import YES
from edc_form_validators import INVALID_ERROR
from edc_form_validators.form_validator import FormValidator


class HospitalizationFormValidator(FormValidator):
    def clean(self):
        self.validate_discharged_date()

        self.required_if(YES, field="have_details", field_required="narrative", inverse=False)

    def validate_discharged_date(self):
        self.required_if(YES, field="discharged", field_required="discharged_date")

        if (
            self.cleaned_data.get("discharged_date")
            and self.cleaned_data.get("admitted_date")
            and (
                self.cleaned_data.get("discharged_date")
                < self.cleaned_data.get("admitted_date")
            )
        ):
            self.raise_validation_error(
                {"discharged_date": "Invalid. Cannot be before date admitted."},
                INVALID_ERROR,
            )

        self.applicable_if(
            YES, field="discharged", field_applicable="discharged_date_estimated"
        )



from dateutil.relativedelta import relativedelta
from django import forms
from django.test import TestCase
from edc_constants.constants import OTHER, UNKNOWN
from edc_constants.disease_constants import BACTERAEMIA
from edc_form_validators import FormValidatorTestCaseMixin
from edc_form_validators.tests.mixins import FormValidatorTestMixin
from edc_utils import get_utcnow

from edc_adverse_event.form_validators import DeathReportFormValidator as Base


class DeathReportFormValidator(FormValidatorTestMixin, Base):
    pass


class TestHospitalizationFormValidation(FormValidatorTestCaseMixin, TestCase):
    @staticmethod
    def get_cleaned_data() -> dict:
        report_datetime = get_utcnow()
        return {
            "report_datetime": report_datetime,
            "death_datetime": report_datetime - relativedelta(days=3),
            "cause_of_death": BACTERAEMIA,
            "cause_of_death_other": "",
            "narrative": "Narrative around death",
        }

    def test_cleaned_data_ok(self):
        cleaned_data = self.get_cleaned_data()
        form_validator = DeathReportFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

    def test_death_datetime_after_report_datetime_raises(self):
        for death_report_date_field in ["death_date", "death_datetime"]:
            for days_after in [1, 3, 14]:
                with self.subTest(
                    death_report_date_field=death_report_date_field, days_after=days_after
                ):
                    report_datetime = get_utcnow()
                    death_datetime = report_datetime + relativedelta(days=days_after)
                    cleaned_data = {
                        "report_datetime": get_utcnow(),
                        death_report_date_field: (
                            death_datetime
                            if death_report_date_field == "death_datetime"
                            else death_datetime.date()
                        ),
                    }
                    form_validator = DeathReportFormValidator(cleaned_data=cleaned_data)
                    form_validator.death_report_date_field = death_report_date_field
                    with self.assertRaises(forms.ValidationError) as cm:
                        form_validator.validate()
                    self.assertIn("report_datetime", cm.exception.error_dict)
                    self.assertIn(
                        "Invalid. Expected a date on or after",
                        str(cm.exception.error_dict.get("report_datetime")),
                    )
                    self.assertIn(
                        "(on or after date of death)",
                        str(cm.exception.error_dict.get("report_datetime")),
                    )

    def test_death_datetime_on_or_before_report_datetime_datetime_ok(self):
        for death_report_date_field in ["death_date", "death_datetime"]:
            for days_before in [0, 1, 2, 14]:
                with self.subTest(
                    death_report_date_field=death_report_date_field, days_before=days_before
                ):
                    report_datetime = get_utcnow()
                    death_datetime = report_datetime - relativedelta(days=days_before)
                    cleaned_data = {
                        "report_datetime": report_datetime,
                        death_report_date_field: (
                            death_datetime
                            if death_report_date_field == "death_datetime"
                            else death_datetime.date()
                        ),
                    }
                    form_validator = DeathReportFormValidator(cleaned_data=cleaned_data)
                    form_validator.death_report_date_field = death_report_date_field
                    try:
                        form_validator.validate()
                    except forms.ValidationError as e:
                        self.fail(f"ValidationError unexpectedly raised. Got {e}")

    def test_cause_of_death_other_required_if_cause_of_death_other(self):
        cleaned_data = self.get_cleaned_data()
        cleaned_data.update(
            {
                "cause_of_death": OTHER,
                "cause_of_death_other": "",
            }
        )
        form_validator = DeathReportFormValidator(cleaned_data=cleaned_data)
        with self.assertRaises(forms.ValidationError) as cm:
            form_validator.validate()
        self.assertIn("cause_of_death_other", cm.exception.error_dict)
        self.assertIn(
            "This field is required.",
            str(cm.exception.error_dict.get("cause_of_death_other")),
        )

        cleaned_data.update({"cause_of_death_other": "Some other cause of death..."})
        form_validator = DeathReportFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

    def test_cause_of_death_other_not_required_if_cause_of_death_not_other(self):
        cleaned_data = self.get_cleaned_data()
        cleaned_data.update(
            {
                "cause_of_death": UNKNOWN,
                "cause_of_death_other": "Some other cause of death",
            }
        )
        form_validator = DeathReportFormValidator(cleaned_data=cleaned_data)
        with self.assertRaises(forms.ValidationError) as cm:
            form_validator.validate()
        self.assertIn("cause_of_death_other", cm.exception.error_dict)
        self.assertIn(
            "This field is not required.",
            str(cm.exception.error_dict.get("cause_of_death_other")),
        )

        cleaned_data.update({"cause_of_death_other": ""})
        form_validator = DeathReportFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")



from .ae_followup import AeFollowupFormValidator
from .ae_initial import AeInitialFormValidator
from .ae_susar import AeSusarFormValidator
from .ae_tmg import AeTmgFormValidator
from .death_report import DeathReportFormValidator
from .death_report_tmg import DeathReportTmgFormValidator
from .hospitalization import HospitalizationFormValidator



from edc_action_item import ActionWithNotification
from edc_constants.constants import DEAD, HIGH_PRIORITY, NO, YES
from edc_reportable import GRADE3, GRADE4, GRADE5

from edc_adverse_event.constants import (
    AE_FOLLOWUP_ACTION,
    AE_INITIAL_ACTION,
    AE_SUSAR_ACTION,
    AE_TMG_ACTION,
    DEATH_REPORT_ACTION,
)

from ..constants import ADVERSE_EVENT_ADMIN_SITE, ADVERSE_EVENT_APP_LABEL


class AeInitialAction(ActionWithNotification):
    name = AE_INITIAL_ACTION
    display_name = "Submit AE Initial Report"
    notification_display_name = "AE Initial Report"
    parent_action_names = []
    reference_model = f"{ADVERSE_EVENT_APP_LABEL}.aeinitial"
    show_link_to_changelist = True
    show_link_to_add = True
    admin_site_name = ADVERSE_EVENT_ADMIN_SITE
    instructions = "Complete the initial AE report"
    priority = HIGH_PRIORITY

    @property
    def deceased(self):
        if (self.reference_obj.ae_grade and self.reference_obj.ae_grade == GRADE5) or (
            self.reference_obj.sae_reason.name and self.reference_obj.sae_reason.name == DEAD
        ):
            return True
        return False

    def get_next_actions(self):
        """Returns next actions."""
        next_actions = []
        next_actions = self.append_ae_followup_next_action(next_actions)
        next_actions = self.append_ae_susar_next_action(next_actions)
        next_actions = self.append_ae_tmg_next_action(next_actions)
        next_actions = self.append_death_report_next_action(next_actions)
        return next_actions

    def append_ae_susar_next_action(self, next_actions=None):
        """Add next AE_SUSAR_ACTION if SUSAR == YES."""
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=AE_SUSAR_ACTION,
            required=(
                self.reference_obj.susar == YES and self.reference_obj.susar_reported == NO
            ),
        )
        return next_actions

    def append_ae_followup_next_action(self, next_actions=None):
        """Add next AeFollowup if not deceased."""
        if not self.deceased:
            next_actions = self.append_to_next_if_required(
                action_name=AE_FOLLOWUP_ACTION, next_actions=next_actions
            )
        return next_actions

    def append_ae_tmg_next_action(self, next_actions=None):
        """add next AE Tmg if G3+SAE, G4, or G5/Death."""
        # add next AeTmgAction if G5/Death
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions, action_name=AE_TMG_ACTION, required=self.deceased
        )
        # add next AeTmgAction if G4
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=AE_TMG_ACTION,
            required=self.reference_obj.ae_grade == GRADE4,
        )
        # add next AeTmgAction if G3 and is an SAE
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=AE_TMG_ACTION,
            required=(self.reference_obj.ae_grade == GRADE3 and self.reference_obj.sae == YES),
        )
        return next_actions

    def append_death_report_next_action(self, next_actions=None):
        """Add next Death report if G5/Death."""
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=DEATH_REPORT_ACTION,
            required=self.deceased,
        )
        return next_actions



from django.db import models
from edc_constants.choices import YES_NO
from edc_model.models import OtherCharField

from ...choices import CONTACT, DEATH_LOCATIONS, INFORMANT


class DeathReportExtraFieldsModelMixin(models.Model):
    """Additional fields for the Death Report model.

    Declare with from `DeathReportModelMixin` from `edc_adverse_event`.
    """

    death_location = models.CharField(
        verbose_name="Where did the participant die?",
        max_length=50,
        choices=DEATH_LOCATIONS,
    )

    death_location_other = OtherCharField()

    hospital_death = models.CharField(
        verbose_name=(
            "Did the participant die in hospital, or die just after " "visiting a hospital?"
        ),
        max_length=50,
        choices=YES_NO,
    )

    hospital_name = models.CharField(
        verbose_name="Which hospital was this", max_length=50, null=True, blank=True
    )

    informant = models.CharField(
        verbose_name="Who has confirmed that the participant has died?",
        max_length=50,
        choices=INFORMANT,
    )

    informant_other = OtherCharField()

    confirmed_by = models.CharField(
        verbose_name="How was this confirmed?",
        max_length=50,
        choices=CONTACT,
    )

    confirmed_by_other = OtherCharField()

    comment = models.TextField(
        verbose_name="Comment",
        null=True,
        blank=True,
        help_text="Provide any additional details, if relevant",
    )

    class Meta:
        abstract = True



from dateutil.relativedelta import relativedelta
from edc_visit_schedule.schedule import Schedule
from edc_visit_schedule.visit import Crf, CrfCollection, Visit
from edc_visit_schedule.visit_schedule import VisitSchedule

from adverse_event_app.consents import consent_v1

crfs = CrfCollection(Crf(show_order=1, model="adverse_event_app.crfone", required=True))

visit = Visit(
    code="1000",
    timepoint=0,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    title="Baseline",
    requisitions=None,
    crfs=crfs,
    requisitions_unscheduled=None,
    crfs_unscheduled=None,
    allow_unscheduled=False,
    facility_name="5-day-clinic",
)

schedule = Schedule(
    name="schedule1",
    onschedule_model="adverse_event_app.onschedule",
    offschedule_model="adverse_event_app.StudyTerminationConclusion",
    appointment_model="edc_appointment.appointment",
    consent_definitions=[consent_v1],
)

visit_schedule = VisitSchedule(
    name="visit_schedule",
    offstudy_model="edc_offstudy.subjectoffstudy",
    death_report_model="adverse_event_app.deathreport",
    locator_model="edc_locator.subjectlocator",
)

schedule.add_visit(visit)

visit_schedule.add_schedule(schedule)



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING

from django.utils.translation import gettext as _
from edc_view_utils import ModelButton

if TYPE_CHECKING:
    from edc_action_item.models import ActionItem
    from edc_model.models import BaseUuidModel

    from edc_adverse_event.model_mixins import (
        AeFollowupModelMixin,
        AeInitialModelMixin,
        DeathReportModelMixin,
        DeathReportTmgModelMixin,
    )

    class AeInitialModel(AeInitialModelMixin, BaseUuidModel): ...

    class AeFollowupModel(AeFollowupModelMixin, BaseUuidModel): ...

    class DeathReportTmgModel(DeathReportTmgModelMixin, BaseUuidModel): ...

    class DeathReportModel(DeathReportModelMixin, BaseUuidModel): ...


@dataclass
class TmgButton(ModelButton):
    model_obj: DeathReportTmgModel | DeathReportModel | AeFollowupModel | AeInitialModel = None
    next_url_name: str | None = field(default="open_tmg_ae_listboard_url")
    only_user_created_may_access: bool | None = None
    forloop_counter: int | None = None
    colors: tuple[str, str, str] = field(default=("warning", "success", "success"))
    titles: tuple[str, str, str] = field(default=(_("Add"), _("Change"), _("View")))
    action_item: ActionItem = (None,)

    disable_all: bool = False

    @property
    def disabled(self) -> str:
        if self.disable_all:
            disabled = "disabled"
        else:
            disabled = super().disabled
            if (
                self.model_obj
                and self.only_user_created_may_access
                and self.model_obj.user_created != self.user.username
            ):
                disabled = "disabled"
        return disabled

    @property
    def fa_icon(self) -> str:
        if self.disabled:
            return "fa-eye-slash"
        return super().fa_icon

    @property
    def btn_id(self) -> str:
        if self.forloop_counter is not None:
            return str(self.forloop_counter)
        return super().btn_id

    @property
    def title(self) -> str:
        if (
            self.model_obj
            and self.only_user_created_may_access
            and self.model_obj.user_created != self.user.username
        ):
            title = _("May only be edited by %(user)s") % {"user": self.model_obj.user_created}
            if self.model_obj.site.id != self.request.site.id:
                title = _("%(title)s when logged into site %(site)s") % {
                    "title": title,
                    "site": self.model_obj.site.id,
                }
        else:
            title = super().title
        return title

    @property
    def label(self) -> str:
        if (
            self.model_obj
            and self.only_user_created_may_access
            and self.model_obj.user_created != self.user.username
        ):
            return _("View")
        return _(super().label)

    @property
    def extra_kwargs(self) -> dict[str, str | int]:
        opts = {}
        if self.action_item.parent_action_item:
            parent_action_item = getattr(self.action_item, "parent_action_item", None)
            if parent_action_item:
                opts.update(
                    parent_action_item=str(parent_action_item.id),
                )
            related_action_item = getattr(self.action_item, "parent_action_item", None)
            if related_action_item:
                opts.update(
                    related_action_item=str(related_action_item.id),
                )
            opts = dict(
                ae_initial=str(self.action_item.parent_action_item.reference_obj.id),
                action_identifier=self.action_item.action_identifier,
                action_item=str(self.action_item.id),
            )
        return opts



from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase, override_settings
from edc_action_item.models.action_item import ActionItem
from edc_constants.constants import CLOSED, NEW, NO, OTHER, YES
from edc_facility.import_holidays import import_holidays
from edc_list_data.site_list_data import site_list_data
from model_bakery import baker

from adverse_event_app import list_data
from edc_adverse_event.constants import DEATH_REPORT_TMG_SECOND_ACTION
from edc_adverse_event.models import CauseOfDeath

from .mixins import DeathReportTestMixin


@override_settings(EDC_LIST_DATA_ENABLE_AUTODISCOVER=False)
class TestDeathReportTmg(DeathReportTestMixin, TestCase):
    @classmethod
    def setUpClass(cls):
        site_list_data.initialize()
        site_list_data.register(list_data, app_name="adverse_event_app")
        site_list_data.load_data()
        import_holidays()
        super().setUpClass()

    def test_death(self):
        # create ae initial
        ae_initial = baker.make_recipe(
            "adverse_event_app.aeinitial",
            subject_identifier=self.subject_identifier,
            susar=YES,
            susar_reported=YES,
            ae_grade=5,
            user_created="erikvw",
        )

        ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            parent_action_item=None,
            action_type__reference_model="adverse_event_app.aeinitial",
        )

        # confirm death report action item is created
        try:
            action_item = ActionItem.objects.get(
                subject_identifier=self.subject_identifier,
                parent_action_item=ae_initial.action_item,
                action_type__reference_model="adverse_event_app.deathreport",
            )
        except ObjectDoesNotExist:
            self.fail("deathreport action unexpectedly does not exist")

        # create death report
        baker.make_recipe(
            "adverse_event_app.deathreport",
            subject_identifier=self.subject_identifier,
            action_identifier=action_item.action_identifier,
            user_created="erikvw",
        )

        # confirm death report action is closed
        action_item = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            parent_action_item=ae_initial.action_item,
            action_type__reference_model="adverse_event_app.deathreport",
        )
        self.assertEqual(action_item.status, CLOSED)

    def test_death_tmg_agrees(self):
        death_report = self.get_death_report()

        # confirm death report tmg action item is created
        try:
            action_item = ActionItem.objects.get(
                subject_identifier=self.subject_identifier,
                related_action_item=death_report.action_item,
                parent_action_item=death_report.action_item,
                action_type__reference_model="adverse_event_app.deathreporttmg",
            )
        except ObjectDoesNotExist:
            self.fail("deathreport action unexpectedly does not exist")

        # create death report TMG
        death_report_tmg = baker.make_recipe(
            "adverse_event_app.deathreporttmg",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item.action_identifier,
            cause_of_death=death_report.cause_of_death,
            cause_of_death_agreed=YES,
            report_status=CLOSED,
            user_created="erikvw",
        )

        self.assertEqual(death_report_tmg.report_status, CLOSED)

        action_item.refresh_from_db()
        self.assertEqual(action_item.status, CLOSED)

    def test_death_tmg_disagrees_still_closes(self):
        death_report = self.get_death_report()

        # confirm death report tmg action item is created
        try:
            action_item = ActionItem.objects.get(
                subject_identifier=self.subject_identifier,
                related_action_item=death_report.action_item,
                parent_action_item=death_report.action_item,
                action_type__reference_model="adverse_event_app.deathreporttmg",
            )
        except ObjectDoesNotExist:
            self.fail("deathreport action unexpectedly does not exist")

        # create death report TMG
        death_report_tmg = baker.make_recipe(
            "adverse_event_app.deathreporttmg",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item.action_identifier,
            cause_of_death=death_report.cause_of_death,
            cause_of_death_agreed=NO,
            report_status=CLOSED,
            user_created="erikvw",
        )

        self.assertEqual(death_report_tmg.report_status, CLOSED)

        action_item.refresh_from_db()
        self.assertEqual(action_item.status, CLOSED)

    def test_death_tmg_disagrees_creates_new_second_tmg_action(self):
        death_report = self.get_death_report()

        action_item = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            parent_action_item=death_report.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmg",
        )

        # create death report TMG
        causes_qs = CauseOfDeath.objects.exclude(name__in=[OTHER, death_report.cause_of_death])
        baker.make_recipe(
            "adverse_event_app.deathreporttmg",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item.action_identifier,
            cause_of_death=causes_qs[0],
            cause_of_death_agreed=NO,
            report_status=CLOSED,
            user_created="erikvw",
        )
        action_item.refresh_from_db()
        self.assertEqual(action_item.status, CLOSED)

        # assert second TMG action is created, because they disagree
        try:
            action_item = ActionItem.objects.get(
                subject_identifier=self.subject_identifier,
                action_type__reference_model="adverse_event_app.deathreporttmgsecond",
            )
        except ObjectDoesNotExist:
            self.fail("deathreporttmgsecond action item unexpectedly does not exist")

        # assert closed
        self.assertEqual(action_item.status, NEW)

    def test_death_tmg_submit_second_tmg_action(self):
        death_report = self.get_death_report()

        action_item = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            parent_action_item=death_report.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmg",
        )

        # create death report TMG
        causes_qs = CauseOfDeath.objects.exclude(name__in=[OTHER, death_report.cause_of_death])
        death_report_tmg = baker.make_recipe(
            "adverse_event_app.deathreporttmg",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item.action_identifier,
            cause_of_death=causes_qs[0],
            cause_of_death_agreed=NO,
            report_status=CLOSED,
            user_created="erikvw",
        )

        death_report.refresh_from_db()

        second_action_item = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            related_action_item=death_report.action_item,
            parent_action_item=death_report_tmg.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmgsecond",
            action_type__name=DEATH_REPORT_TMG_SECOND_ACTION,
        )

        # create 2nd death report TMG
        death_report_tmg_second = baker.make_recipe(
            "adverse_event_app.deathreporttmgsecond",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=second_action_item.action_identifier,
            cause_of_death=causes_qs[0],
            cause_of_death_agreed=NO,
            report_status=CLOSED,
            user_created="erikvw",
        )

        second_action_item.refresh_from_db()
        death_report_tmg_second.save()
        self.assertEqual(second_action_item.status, CLOSED)

        death_report_tmg_second.save()
        second_action_item.refresh_from_db()
        self.assertEqual(second_action_item.status, CLOSED)

    def test_death_tmg_disagrees_tmg_action_changes(self):
        """Assert changes to death report tmg are safe.

        For example: (cause of death agreed = YES/NO)
        """

        death_report = self.get_death_report()

        action_item = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            parent_action_item=death_report.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmg",
        )

        # create death report TMG
        causes_qs = CauseOfDeath.objects.exclude(name__in=[OTHER, death_report.cause_of_death])
        death_report_tmg = baker.make_recipe(
            "adverse_event_app.deathreporttmg",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item.action_identifier,
            cause_of_death=causes_qs[0],
            cause_of_death_agreed=YES,
            report_status=CLOSED,
            user_created="erikvw",
        )

        self.assertRaises(
            ObjectDoesNotExist,
            ActionItem.objects.get,
            subject_identifier=self.subject_identifier,
            related_action_item=death_report.action_item,
            parent_action_item=death_report_tmg.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmgsecond",
            action_type__name=DEATH_REPORT_TMG_SECOND_ACTION,
        )

        death_report_tmg.cause_of_death_agreed = NO
        death_report_tmg.save()

        ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            related_action_item=death_report.action_item,
            parent_action_item=death_report_tmg.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmgsecond",
            action_type__name=DEATH_REPORT_TMG_SECOND_ACTION,
        )

        death_report_tmg.cause_of_death_agreed = YES
        death_report_tmg.save()

        self.assertRaises(
            ObjectDoesNotExist,
            ActionItem.objects.get,
            subject_identifier=self.subject_identifier,
            related_action_item=death_report.action_item,
            parent_action_item=death_report_tmg.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmgsecond",
            action_type__name=DEATH_REPORT_TMG_SECOND_ACTION,
        )

    def test_tmg_once_closed_does_not_repopen(self):
        death_report = self.get_death_report()

        action_item = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            parent_action_item=death_report.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmg",
        )

        causes_qs = CauseOfDeath.objects.exclude(name__in=[OTHER, death_report.cause_of_death])
        death_report_tmg = baker.make_recipe(
            "adverse_event_app.deathreporttmg",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item.action_identifier,
            cause_of_death=causes_qs[0],
            cause_of_death_agreed=YES,
            report_status=CLOSED,
            user_created="erikvw",
        )

        action_item.refresh_from_db()
        self.assertEqual(action_item.status, CLOSED)

        death_report_tmg.cause_of_death_agreed = NO
        death_report_tmg.save()

        action_item.refresh_from_db()
        self.assertEqual(action_item.status, CLOSED)

    def test_2nd_tmg_once_closed_does_not_repopen(self):
        death_report = self.get_death_report()

        action_item = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            parent_action_item=death_report.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmg",
        )

        causes_qs = CauseOfDeath.objects.exclude(name__in=[OTHER, death_report.cause_of_death])
        death_report_tmg = baker.make_recipe(
            "adverse_event_app.deathreporttmg",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item.action_identifier,
            cause_of_death=causes_qs[0],
            cause_of_death_agreed=NO,
            report_status=CLOSED,
            user_created="erikvw",
        )

        action_item_two = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            related_action_item=death_report.action_item,
            parent_action_item=death_report_tmg.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmgsecond",
            action_type__name=DEATH_REPORT_TMG_SECOND_ACTION,
        )

        # create 2nd death report TMG
        death_report_tmg_second = baker.make_recipe(
            "adverse_event_app.deathreporttmgsecond",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item_two.action_identifier,
            cause_of_death=causes_qs[0],
            cause_of_death_agreed=NO,
            report_status=CLOSED,
            user_created="erikvw",
        )

        action_item_two.refresh_from_db()
        self.assertEqual(action_item_two.status, CLOSED)

        death_report_tmg_second.cause_of_death_agreed = YES
        death_report_tmg_second.save()

        action_item_two.refresh_from_db()
        self.assertEqual(action_item_two.status, CLOSED)

    def test_delete_second_tmg_and_change_agreed_to_yes(self):
        death_report = self.get_death_report()

        action_item_one = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            parent_action_item=death_report.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmg",
        )

        causes_qs = CauseOfDeath.objects.exclude(name__in=[OTHER, death_report.cause_of_death])
        death_report_tmg = baker.make_recipe(
            "adverse_event_app.deathreporttmg",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item_one.action_identifier,
            cause_of_death=causes_qs[0],
            cause_of_death_agreed=NO,
            report_status=CLOSED,
            user_created="erikvw",
        )

        action_item_two = ActionItem.objects.get(
            subject_identifier=self.subject_identifier,
            related_action_item=death_report.action_item,
            parent_action_item=death_report_tmg.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmgsecond",
            action_type__name=DEATH_REPORT_TMG_SECOND_ACTION,
        )

        # create 2nd death report TMG
        death_report_tmg_second = baker.make_recipe(
            "adverse_event_app.deathreporttmgsecond",
            death_report=death_report,
            subject_identifier=self.subject_identifier,
            action_identifier=action_item_two.action_identifier,
            cause_of_death=causes_qs[0],
            cause_of_death_agreed=NO,
            report_status=CLOSED,
            user_created="erikvw",
        )

        action_item_one.refresh_from_db()
        self.assertEqual(action_item_one.status, CLOSED)
        action_item_two.refresh_from_db()
        self.assertEqual(action_item_two.status, CLOSED)

        # delete 2nd TMG report
        death_report_tmg_second.delete()

        # assert status for 2nd TMG action item is reset to NEW
        action_item_two.refresh_from_db()
        self.assertEqual(action_item_two.status, NEW)

        # change 1st TMG report to agrees
        death_report_tmg.cause_of_death_agreed = YES
        death_report_tmg.save()

        # action item for 2nd tmg is auto-deleted
        self.assertRaises(
            ObjectDoesNotExist,
            ActionItem.objects.get,
            subject_identifier=self.subject_identifier,
            related_action_item=death_report.action_item,
            parent_action_item=death_report_tmg.action_item,
            action_type__reference_model="adverse_event_app.deathreporttmgsecond",
            action_type__name=DEATH_REPORT_TMG_SECOND_ACTION,
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Optional
from zoneinfo import ZoneInfo

from django import forms
from django.apps import apps as django_apps
from django.conf import settings
from edc_constants.constants import CLOSED, OTHER
from edc_form_validators.base_form_validator import INVALID_ERROR
from edc_utils import convert_php_dateformat

if TYPE_CHECKING:
    from datetime import date


class DeathReportFormValidatorMixin:
    """A form validator mixin for the Death Report form"""

    death_report_date_field = "death_datetime"
    study_day_field = "study_day"

    @property
    def cause_of_death_model_cls(self):
        return django_apps.get_model("edc_adverse_event.causeofdeath")

    def clean(self: Any) -> None:
        self.validate_study_day_with_death_report_date()

        self.date_is_after_or_raise(
            field="report_datetime",
            reference_value=self.death_report_date,
            inclusive=True,
            extra_msg="(on or after date of death)",
        )

        cause_of_death = self.cause_of_death_model_cls.objects.get(name=OTHER)

        self.validate_other_specify(
            field="cause_of_death",
            other_specify_field="cause_of_death_other",
            other_stored_value=cause_of_death.name,
        )

        self.required_if(
            CLOSED, field="report_status", field_required="report_closed_datetime"
        )

    @property
    def death_report_date(self: Any) -> date:
        try:
            return self.cleaned_data.get(self.death_report_date_field).date()
        except AttributeError:
            return self.cleaned_data.get(self.death_report_date_field)

    def validate_study_day_with_death_report_date(
        self: Any,
        subject_identifier: Optional[str] = None,
    ) -> None:
        """Raises an exception if study day does not match
        calculation against ZoneInfo.

        Note: study-day is 1-based.

        This is skipped if `study_day_field` does not exist.
        """
        study_day = self.cleaned_data.get(self.study_day_field)
        if study_day is not None and self.death_report_date is not None:
            subject_identifier = (
                subject_identifier
                or self.cleaned_data.get("subject_identifier")
                or self.instance.subject_identifier
            )
            if not subject_identifier:
                raise ValueError(f"Subject identifier cannot be None. See {repr(self)}")
            registered_subject_model_cls = django_apps.get_model(
                "edc_registration.registeredsubject"
            )
            randomization_datetime = registered_subject_model_cls.objects.get(
                subject_identifier=subject_identifier
            ).randomization_datetime
            days_on_study = (self.death_report_date - randomization_datetime.date()).days
            if study_day - 1 != days_on_study:
                formatted_date = randomization_datetime.astimezone(
                    ZoneInfo(settings.TIME_ZONE)
                ).strftime(convert_php_dateformat(settings.DATETIME_FORMAT))
                message = {
                    self.study_day_field: (
                        f"Invalid. Expected {days_on_study + 1}. "
                        f"Subject was registered on {formatted_date}"
                    )
                }
                raise forms.ValidationError(message, code=INVALID_ERROR)



from django import forms
from django.core.exceptions import ValidationError
from django.test import TestCase, override_settings
from edc_constants.constants import NO, NOT_APPLICABLE, OTHER, YES
from edc_form_validators import NOT_REQUIRED_ERROR
from edc_list_data.site_list_data import site_list_data
from edc_sites.tests import SiteTestCaseMixin

from adverse_event_app import list_data
from edc_adverse_event.constants import AE_WITHDRAWN
from edc_adverse_event.form_validators import (
    AeFollowupFormValidator,
    AeInitialFormValidator,
    AeTmgFormValidator,
)
from edc_adverse_event.models import AeClassification, SaeReason


@override_settings(EDC_LIST_DATA_ENABLE_AUTODISCOVER=False)
class TestFormValidators(SiteTestCaseMixin, TestCase):
    @classmethod
    def setUpClass(cls):
        site_list_data.initialize()
        site_list_data.register(list_data, app_name="adverse_event_app")
        site_list_data.load_data()
        super().setUpClass()

    def test_ae_cause_yes(self):
        options = {"ae_cause": YES, "ae_cause_other": None}
        form_validator = AeInitialFormValidator(cleaned_data=options)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("ae_cause_other", form_validator._errors)

    def test_ae_cause_no(self):
        cleaned_data = {"ae_cause": NO, "ae_cause_other": YES}
        form_validator = AeInitialFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError:
            pass
        self.assertIn("ae_cause_other", form_validator._errors)
        self.assertIn(NOT_REQUIRED_ERROR, form_validator._error_codes)

    def test_sae_reason_not_applicable(self):
        sae_reason = SaeReason.objects.get(name=NOT_APPLICABLE)
        cleaned_data = {"sae": YES, "sae_reason": sae_reason}
        form_validator = AeInitialFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("sae_reason", form_validator._errors)

    def test_susar_reported_not_applicable(self):
        cleaned_data = {"susar": YES, "susar_reported": NOT_APPLICABLE}
        form_validator = AeInitialFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("susar_reported", form_validator._errors)

    def test_ae_tmg_reported_ae_classification(self):
        ae_classification = AeClassification.objects.get(name=OTHER)
        cleaned_data = {
            "investigator_ae_classification": ae_classification,
            "investigator_ae_classification_other": None,
        }
        form_validator = AeTmgFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("investigator_ae_classification_other", form_validator._errors)

        cleaned_data = {
            "investigator_ae_classification": ae_classification,
            "investigator_ae_classification_other": None,
        }
        form_validator = AeTmgFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("investigator_ae_classification_other", form_validator._errors)

    def test_ae_followup(self):
        cleaned_data = {"outcome": None, "followup": None}
        form_validator = AeFollowupFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

        cleaned_data = {"outcome": AE_WITHDRAWN, "followup": None}
        form_validator = AeFollowupFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

        cleaned_data = {"outcome": AE_WITHDRAWN, "followup": NO}
        form_validator = AeFollowupFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

        cleaned_data = {"outcome": AE_WITHDRAWN, "followup": YES}
        form_validator = AeFollowupFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("followup", form_validator._errors)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from edc_constants.constants import CLOSED, NEW, OPEN
from edc_dashboard.view_mixins import EdcViewMixin
from edc_listboard.view_mixins import ListboardFilterViewMixin, SearchFormViewMixin
from edc_listboard.views import ListboardView as BaseListboardView
from edc_navbar import NavbarViewMixin
from edc_navbar.get_default_navbar import get_default_navbar
from edc_utils import get_utcnow

from ...constants import AE_TMG_ACTION
from ...utils import get_adverse_event_app_label, has_valid_tmg_perms

if TYPE_CHECKING:
    from django.db.models import Q


class TmgAeListboardViewMixin(
    NavbarViewMixin,
    EdcViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
    BaseListboardView,
):
    listboard_back_url = "tmg_home_url"

    ae_tmg_model = f"{get_adverse_event_app_label()}.aetmg"
    listboard_template = "tmg_ae_listboard_template"
    listboard_url = "tmg_ae_listboard_url"
    listboard_panel_style = "warning"
    listboard_model = "edc_action_item.actionitem"
    listboard_panel_title = "TMG: AE Reports"
    listboard_view_permission_codename = "edc_adverse_event.view_tmg_listboard"

    navbar_name = get_default_navbar()
    navbar_selected_item = "tmg_home"
    ordering = "-report_datetime"
    paginate_by = 10
    search_form_url = "tmg_ae_listboard_url"
    action_type_names = [AE_TMG_ACTION]

    search_fields = [
        "subject_identifier",
        "action_identifier",
        "parent_action_item__action_identifier",
        "related_action_item__action_identifier",
        "user_created",
        "user_modified",
    ]

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        kwargs.update(
            AE_TMG_ACTION=AE_TMG_ACTION,
            utc_date=get_utcnow().date(),
            subject_identifier=self.kwargs.get("subject_identifier"),
        )
        return super().get_context_data(**kwargs)

    def get_queryset_filter_options(self, request, *args, **kwargs) -> tuple[Q, dict]:
        q_object, options = super().get_queryset_filter_options(request, *args, **kwargs)
        options.update(
            action_type__name__in=self.action_type_names,
            status__in=[NEW, OPEN, CLOSED],
        )
        if kwargs.get("subject_identifier"):
            options.update({"subject_identifier": kwargs.get("subject_identifier")})
        return q_object, options


class StatusTmgAeListboardView(TmgAeListboardViewMixin):
    status = None

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        has_valid_tmg_perms(self.request, add_message=True)
        kwargs.update(status=self.status)
        return super().get_context_data(**kwargs)

    def get_queryset_filter_options(self, request, *args, **kwargs) -> tuple[Q, dict]:
        q_object, options = super().get_queryset_filter_options(request, *args, **kwargs)
        options.update({"status": self.status})
        return q_object, options



from dateutil.relativedelta import relativedelta
from django.conf import settings
from edc_consent.consent_definition import ConsentDefinition
from edc_consent.site_consents import site_consents
from edc_constants.constants import FEMALE, MALE
from edc_utils import get_utcnow

consent_v1 = ConsentDefinition(
    "adverse_event_app.subjectconsentv1",
    version="1",
    start=getattr(
        settings,
        "EDC_PROTOCOL_STUDY_OPEN_DATETIME",
        get_utcnow().replace(microsecond=0, second=0, minute=0, hour=0)
        - relativedelta(months=1),
    ),
    end=getattr(
        settings,
        "EDC_PROTOCOL_STUDY_CLOSE_DATETIME",
        get_utcnow().replace(microsecond=999999, second=59, minute=59, hour=11)
        + relativedelta(years=1),
    ),
    age_min=18,
    age_is_adult=18,
    age_max=110,
    gender=[MALE, FEMALE],
)

site_consents.register(consent_v1)



from django.db import models
from edc_constants.constants import NOT_APPLICABLE


class AeFollowupMethodsModelMixin(models.Model):
    def __str__(self):
        return f"{self.action_identifier[-9:]}"

    def save(self, *args, **kwargs):
        self.subject_identifier = self.ae_initial.subject_identifier
        super().save(*args, **kwargs)

    def natural_key(self):
        return (self.action_identifier,)

    @property
    def report_date(self):
        """Returns a date based on the UTC datetime."""
        return self.report_datetime.date()

    @property
    def severity(self):
        if self.ae_grade == NOT_APPLICABLE:
            return "unchanged"
        return self.ae_grade

    def get_action_item_reason(self):
        return self.ae_initial.ae_description

    class Meta:
        abstract = True



from django import forms
from edc_constants.constants import DEAD, YES
from edc_reportable import GRADE5

from ..form_validators import AeFollowupFormValidator
from ..utils import validate_ae_initial_outcome_date
from .ae_modelform_mixin import AeModelFormMixin


class AeFollowupModelFormMixin(AeModelFormMixin):
    form_validator_cls = AeFollowupFormValidator

    def clean(self):
        cleaned_data = super().clean()
        validate_ae_initial_outcome_date(self)
        self.validate_no_followup_upon_death()
        return cleaned_data

    def validate_no_followup_upon_death(self):
        if self.cleaned_data.get("followup") == YES:
            if (
                self.cleaned_data.get("ae_grade") == GRADE5
                or self.cleaned_data.get("outcome") == DEAD
            ):
                raise forms.ValidationError(
                    {
                        "followup": (
                            "Expected No. Submit a death report when the "
                            "severity increases to grade 5."
                        )
                    }
                )

    class Meta:
        help_text = {"subject_identifier": "(read-only)", "action_identifier": "(read-only)"}
        widgets = {
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



from edc_form_validators import FormValidator

from ..form_validator_mixins import DeathReportFormValidatorMixin


class DeathReportFormValidator(DeathReportFormValidatorMixin, FormValidator):
    death_report_date_field = "death_datetime"



from __future__ import annotations

from django.contrib import admin
from django.contrib.admin import display
from django.template.loader import render_to_string
from django_audit_fields.admin import audit_fieldset_tuple
from edc_action_item import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ActionItemModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..forms import AeFollowupForm
from ..templatetags.edc_adverse_event_extras import (
    format_ae_followup_description,
    select_description_template,
)
from .modeladmin_mixins import AdverseEventModelAdminMixin, NonAeInitialModelAdminMixin


class AeFollowupModelAdminMixin(
    ModelAdminSubjectDashboardMixin,
    NonAeInitialModelAdminMixin,
    AdverseEventModelAdminMixin,
    ActionItemModelAdminMixin,
):
    form = AeFollowupForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "ae_initial",
                    "report_datetime",
                    "outcome_date",
                    "outcome",
                    "ae_grade",
                    "relevant_history",
                    "followup",
                )
            },
        ),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "outcome": admin.VERTICAL,
        "followup": admin.VERTICAL,
        "ae_grade": admin.VERTICAL,
    }

    def get_search_fields(self, request) -> tuple[str, ...]:
        search_fields = super().get_search_fields(request)
        return tuple(
            set(
                search_fields
                + (
                    "action_identifier",
                    "ae_initial__subject_identifier",
                    "ae_initial__action_identifier",
                )
            )
        )

    def get_list_display(self, request) -> tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = (
            "subject_identifier_column",
            "dashboard",
            "description_column",
            "documents_column",
        )
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("ae_grade", "followup", "outcome_date", "outcome", "report_datetime")
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)

    @display(description="Description", ordering="-report_datetime")
    def description_column(self, obj):
        """Returns a formatted comprehensive description of the SAE
        combining multiple fields.
        """
        context = format_ae_followup_description({}, obj, None)
        return render_to_string(select_description_template("aefollowup"), context)



from django.utils.html import format_html
from django.utils.safestring import mark_safe
from edc_action_item.action_with_notification import ActionWithNotification
from edc_action_item.site_action_items import site_action_items
from edc_constants.constants import DEAD, HIGH_PRIORITY, YES
from edc_ltfu.constants import LOST_TO_FOLLOWUP
from edc_notification.utils import get_email_contacts
from edc_reportable.constants import GRADE5
from edc_visit_schedule.utils import (
    OnScheduleError,
    get_offschedule_models,
    get_onschedule_models,
)

from edc_adverse_event.constants import (
    AE_FOLLOWUP_ACTION,
    AE_INITIAL_ACTION,
    DEATH_REPORT_ACTION,
)

from ..constants import ADVERSE_EVENT_ADMIN_SITE, ADVERSE_EVENT_APP_LABEL


class AeFollowupAction(ActionWithNotification):
    """
    For example:
        from edc_adverse_event.action_items import AeFollowupAction

        site_action_items.register(AeInitialAction)

    """

    reference_model = f"{ADVERSE_EVENT_APP_LABEL}.aefollowup"
    related_reference_model = f"{ADVERSE_EVENT_APP_LABEL}.aeinitial"

    name = AE_FOLLOWUP_ACTION
    display_name = "Submit AE Followup Report"
    notification_display_name = "AE Followup Report"
    parent_action_names = [AE_INITIAL_ACTION, AE_FOLLOWUP_ACTION]
    related_reference_fk_attr = "ae_initial"
    create_by_user = False
    show_link_to_changelist = True
    admin_site_name = ADVERSE_EVENT_ADMIN_SITE
    instructions = format_html(  # nosec B703, B308
        "Upon submission the TMG group will be notified "
        'by email at <a href="mailto:{}">{}</a>',
        mark_safe(get_email_contacts("tmg") or "#"),  # nosec B703, B308
        mark_safe(get_email_contacts("tmg") or "unknown"),  # nosec B703, B308
    )
    priority = HIGH_PRIORITY

    def get_next_actions(self):
        next_actions = []

        # add AE followup to next_actions if followup.
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=self.name,
            required=self.reference_obj.followup == YES,
        )

        # add Death report to next_actions if G5/Death
        next_actions = self.append_to_next_if_required(
            next_actions=next_actions,
            action_name=DEATH_REPORT_ACTION,
            required=(
                self.reference_obj.outcome == DEAD or self.reference_obj.ae_grade == GRADE5
            ),
        )

        next_actions = self.update_next_actions_ltfu(next_actions)
        return next_actions

    @property
    def offschedule_models(self):
        """Returns a list of offschedule models, in label_lower format."""
        return get_offschedule_models(
            subject_identifier=self.subject_identifier,
            report_datetime=self.reference_obj.report_datetime,
        )

    @property
    def onschedule_models(self):
        """Returns a list of offschedule models, in label_lower format."""
        return get_onschedule_models(
            subject_identifier=self.subject_identifier,
            report_datetime=self.reference_obj.report_datetime,
        )

    def update_next_actions_ltfu(self, next_actions=None):
        """Add Study termination to next_actions if LTFU."""
        if self.reference_obj.outcome and self.reference_obj.outcome == LOST_TO_FOLLOWUP:
            if not self.onschedule_models:
                raise OnScheduleError(
                    f"Subject cannot be lost to followup. "
                    f"Subject is not on schedule! Got {self.subject_identifier}."
                )
            for offschedule_model in self.offschedule_models:
                action_cls = site_action_items.get_by_model(model=offschedule_model)
                next_actions = self.append_to_next_if_required(
                    next_actions=next_actions, action_cls=action_cls, required=True
                )
        return next_actions



from .death_listboard_view import DeathListboardView
from .home_view import TmgHomeView
from .status_listboards import (
    ClosedTmgAeListboardView,
    NewTmgAeListboardView,
    OpenTmgAeListboardView,
)
from .summary_listboard_view import SummaryListboardView
